function getMuscAttachBatch(boneName)
% getMuscAttachBatch finds geometric centre of attachment for muscles
% on bone 'boneName' (all .obj files). BATCH VERSION, assumes all non-bone 
% objs in folder are muscles to process. Use input 'SET MAX DISTANCE FOR CONTACT: '
% to control estimated contact area: all vertices closer than this distance
% will be considered to represent the contact area between the muscle and
% bone. Setting orientMode = 1 uses the normal of a plane
% fitted to the attachment area to estimate contact centroid, 2 uses the
% long-axis of the muscle instead (good for tendon attachments).

%% load bone

[bV,bF] = loadWobj(boneName);

%% get obj list
% assumes all other obj files in this folder are muscles
folderStruct = dir('*.obj');

% convert struct to list
objList = cell(size(folderStruct,1),1);
for loop1 = 1:size(folderStruct,1)
    objList(loop1) = {folderStruct(loop1).name};
end

clear folderStruct

% remove bone from muscle list
objList(strcmp(objList,boneName)) = [];

for outerLoop = 1:length(objList)
    muscName = cell2mat(objList(outerLoop));
    
    % declare job
    display(['CURRENT PROCESSING ATTACHMENTS FOR ', muscName])
    
    % load bone and muscle .objs
    [mV,mF] = loadWobj(muscName);
    
    %% first get list of min distances of each vert in muscle to closest vert in
    % bone
    minDists = zeros(size(mV,1),1);
    for loop1 = 1:size(mV,1)
        currV = mV(loop1,:);
        minDists(loop1) = min(sqrt(sum(abs(bV - repmat(currV,size(bV,1),1)).^2,2)));
    end
    
    % setting vTol (vertex closeness tolerance for counting as 'contact'), this
    % bit is tricky. Lets go with anything within two of the minimum distance
    % not working, lets do it with inputs and a loop.
    
    
    
    vTol = min(minDists);
    cntctV = mV(minDists<=vTol,:);
    
    
    figure(1)
    clf
    subplot(1,2,1)
    title('MUSCLE/BONE CONTACT')
    hold on
    trimesh(bF,bV(:,1), bV(:,2), bV(:,3),'edgecolor',[0.6 0.6 0.6], 'facecolor',[0.9 0.9 0.9]);
    plot3(mV(:,1), mV(:,2), mV(:,3),'.','color','k','markersize',1);
    cntcPlot = plot3(cntctV(:,1), cntctV(:,2), cntctV(:,3),'.r');
    axis equal
    view(3)
    
    subplot(1,2,2)
    title('ATTACHMENT  CENTROID')
    hold on
    trimesh(bF,bV(:,1), bV(:,2), bV(:,3),'edgecolor',[0.6 0.6 0.6], 'facecolor',[0.9 0.9 0.9]);
    cntcPlot2 = plot3(cntctV(:,1), cntctV(:,2), cntctV(:,3),'.r');
    attachPlot = plot3(NaN,NaN,NaN,'go','markersize',15,'markerfacecolor','green');
    axis equal
    view(3)
    
    %%
    doLoop = true;
    doLoop2 = true;
    while doLoop
        
        while doLoop2
            
            vTol = input('SET MIN DISTANCE FOR CONTACT: ');
            
            % get contact area and centroid
            cntctVDex = (1:size(mV,1))';
            cntctVDex = cntctVDex(minDists<=vTol);
            cntctV = mV(cntctVDex,:);
            
            % draw results
            set(cntcPlot,'XData',cntctV(:,1),'YData',cntctV(:,2),'ZData',cntctV(:,3))
            set(cntcPlot2,'XData',cntctV(:,1),'YData',cntctV(:,2),'ZData',cntctV(:,3))
            doLoop2Choice = input('HAPPY WITH CONTACT? y/n: ','s');
            
            if doLoop2Choice=='y' || doLoop2Choice=='Y'
                doLoop2 = false;
            end
            
        end
        
        % choose complex or simple mode
        runMode = input('CHOOSE MODE 1 (for small attachments) OR MODE 2 (for broad attachments): ');
        
        if runMode==1
            
            % make vector from mean bone point cloud to mean contact point cloud
            bonePntCent = mean(bV);
            contactCent = mean(cntctV);
            attachRay = [bonePntCent; contactCent];
            
        elseif runMode==2
            
            cent = getPointCloudCentroid(cntctV);
            
            % get contact faces
            cntctF = zeros(1,size(mF,2));
            for loop1 = 1:size(cntctV,1)
                
                [cntctFdex,~] = find(mF==cntctVDex(loop1));
                cntctF = [cntctF; mF(cntctFdex,:)];
                
            end
            cntctF(1,:) = [];
            
            % NB because we need faces, and I can't be arsed to work out how to
            % reorder a matrix based on another one, we're going to use all of
            % muscle V for the following
            
            % generate rays, sphere generates unit vectors
            [sphrX,sphrY,sphrZ] = sphere(20);
            
            % unroll into column vectors
            rays = [sphrX(1:end)' sphrY(1:end)' sphrZ(1:end)'];
            
            % scale rays to max dim of pointcloud for vis
            maxDim = max([max(cntctV(:,1)-min(cntctV(:,1))) max(cntctV(:,2)-min(cntctV(:,2))) max(cntctV(:,3)-min(cntctV(:,3)))]);
            scaleFactor = maxDim/max(max(rays));
            rays = rays.*scaleFactor;
            
            % prep vis (debug)
            % figure(1)
            % clf
            % hold on
            
            % prep storage
            rayIntersects = zeros(1,3);
            
            for loop1 = 1:size(rays,1)
                % get ray as vector from centroid of point cloud
                currRay = [cent; cent+rays(loop1,:)];
                currInter = getRayMeshIntersect(mV,cntctF,currRay);
                
                if sum(isnan(currInter))~=3
                    rayIntersects = [rayIntersects; currInter];
                    % check results (debug)
                    
                    %         trimesh(f,v(:,1), v(:,2), v(:,3),'edgecolor','r', 'facecolor','none');
                    %         plot3(currInter(:,1),currInter(:,2),currInter(:,3),'k*')
                    %         plot3(currRay(:,1),currRay(:,2),currRay(:,3),'-b')
                    %         currRay2 = [cent; cent-rays(loop1,:)];
                    %         plot3(currRay2(:,1),currRay2(:,2),currRay2(:,3),'-b')
                    %         axis equal
                    %         pause(0.001);
                end
                
            end
            
            % remove first value
            rayIntersects(1,:) = [];
            
            
            % make all unit vectors originating from cent
            rayIntersectsZero = rayIntersects - repmat(cent,size(rayIntersects,1),1);
            rayNorms = sqrt(sum(rayIntersectsZero.^2,2));
            rayIntersectsUnit = rayIntersectsZero./[rayNorms rayNorms rayNorms];
            
            % get mean
            attachRay = [cent; cent+(mean(rayIntersectsUnit,1)*scaleFactor)];
            
        end
        
        % get attachment of mean (NB BUG - this may not contact actual
        % attachment site, so use bone surface instead...)
        boneAttach = getRayMeshIntersect(bV,bF,attachRay);
        
        % you'll probably get multiple contacts, so use the one closest to the
        % cntct point cloud
        minDistsAttach = [(1:size(boneAttach,1))' zeros(size(boneAttach,1),1)];
        for loop1 = 1:size(boneAttach,1)
            
            minDistsAttach(loop1,2) = min(sqrt(sum(abs(cntctV - repmat(boneAttach(loop1,:),size(cntctV,1),1)).^2,2)));
            
        end
        minDistsAttach = sortrows(minDistsAttach,2);
        attach = boneAttach(minDistsAttach(1,1),:);
        
        % draw results
        set(attachPlot,'XData',attach(:,1),'YData',attach(:,2),'ZData',attach(:,3))
        
        doLoopChoice = input('HAPPY WITH ATTACHMENT? y/n: ','s');
        
        
        if doLoopChoice=='y' || doLoopChoice=='Y'
            doLoop = false;
        else
            doLoop2 = true;
        end
        
    end
    
    %% output
    
    % make name
    outName = [regexprep(muscName,'.obj',''),'_AttachTo_',regexprep(boneName,'.obj','')];
    
    % output
    save([outName,'.mat'], 'attach');
    csvwrite([outName,'.csv'],attach);
    saveas(1,[outName,'.fig']);
    
end

close(1)

end

%% embedded functions

function [cent] = getPointCloudCentroid(pnts)

% EXTRA SETTINGS
identity_threshold = 1/100000; % this sets how close intersect points can be
% before they get merged to avoid duplications arising from rare exact
% ray/edge interactions. Due to rounding errors (probably), 'unique' does
% not catch all of these duplications

% set ray parameter
fidelity = 50;

% set density to 1
density=1;

%% use qhull to skin point cloud
faces = convhulln(pnts);

%% GET RAY PARAMETERS

% determine body dimensions
body_box = [max(pnts(:,1)) max(pnts(:,2)) max(pnts(:,3)); ...
    min(pnts(:,1)) min(pnts(:,2)) min(pnts(:,3))];
body_dims = [(body_box(1,1)-body_box(2,1)) (body_box(1,2)-body_box(2,2)) (body_box(1,3)-body_box(2,3))];

% get longest dimension
[~,ray_dim] = max(body_dims);

% check for equally large dimensions, use first max equal dimension if
% found
if sum(body_dims==body_dims(ray_dim))~=1
    ray_dim = find((body_dims==ray_dim)==1, 1,'first');
end

%% BUILD FIDELITY * FIDELITY (2D) ARRAY OF RAYS POINTING ALONG MAX DIMENSION

array_dims = [1 2 3];
array_dims(ray_dim) = [];

array_1_unit = body_dims(array_dims(1))/(fidelity);
array_1_start = body_box(2,array_dims(1)) + (array_1_unit/2);
array_1_stop = body_box(1,array_dims(1)) - (array_1_unit/2);
array_1 = array_1_start:array_1_unit:array_1_stop;
% EXCEPTION CATCH - rounding error can sometimes cause array setup to stop
% before reaching proper stop point
if length(array_1)<fidelity
    array_1 = [array_1 array_1_unit];
end

array_2_unit = body_dims(array_dims(2))/(fidelity);
array_2_start = body_box(2,array_dims(2)) + (array_2_unit/2);
array_2_stop = body_box(1,array_dims(2)) - (array_2_unit/2);
array_2 = array_2_start:array_2_unit:array_2_stop;
% EXCEPTION CATCH - rounding error can sometimes cause array setup to stop
% before reaching proper stop point
if length(array_2)<fidelity
    array_2 = [array_2 array_2_unit];
end

rays = zeros(fidelity^2,2);

loop1_count = 1;
for loop1 = fidelity:fidelity:fidelity^2
    rays(loop1-fidelity+1:loop1,1) = array_1;
    rays(loop1-fidelity+1:loop1,2) = array_2(loop1_count);
    loop1_count = loop1_count+1;
end

%% MAIN LOOP: GO THROUGH RAYS, FIND POLY INTERSECTIONS, POPULATE VOLUME

% intialise element mass storage
element_masses = 0;

% intialise element CoM storage
element_CoMs = [0 0 0];

% get polygon bounding boxes (makes searching faster
f_ids = 1:size(faces,1);
searchable_f = [f_ids' faces];
if sum(sum(faces==0))>0;
    % if compound mesh, process tris and quads seperately
    % get tris
    trindex = searchable_f(:,end)==0;
    tris = searchable_f(trindex,:);
    tris(:,end) = [];
    % get tri verts
    tri_verts = zeros(size(tris,1),9);
    tri_verts(:,1:3) = pnts(tris(:,2),:);
    tri_verts(:,4:6) = pnts(tris(:,3),:);
    tri_verts(:,7:9) = pnts(tris(:,4),:);
    % get tri vert max mins
    tri_maxX = max(tri_verts(:,1:3:end),[],2);
    tri_maxY = max(tri_verts(:,2:3:end),[],2);
    tri_maxZ = max(tri_verts(:,3:3:end),[],2);
    tri_minX = min(tri_verts(:,1:3:end),[],2);
    tri_minY = min(tri_verts(:,2:3:end),[],2);
    tri_minZ = min(tri_verts(:,3:3:end),[],2);
    % assemble as bounding box
    tri_boxes = [tris(:,1) tri_maxX tri_maxY tri_maxZ tri_minX tri_minY...
        tri_minZ];
    
    % get quads
    quadex = searchable_f(:,end)>0;
    quads = searchable_f(quadex,:);
    % get quad verts
    quad_verts = zeros(size(quads,1),12);
    quad_verts(:,1:3) = pnts(quads(:,2),:);
    quad_verts(:,4:6) = pnts(quads(:,3),:);
    quad_verts(:,7:9) = pnts(quads(:,4),:);
    quad_verts(:,10:12) = pnts(quads(:,5),:);
    % get quad vert max mins
    quad_maxX = max(quad_verts(:,1:3:end),[],2);
    quad_maxY = max(quad_verts(:,2:3:end),[],2);
    quad_maxZ = max(quad_verts(:,3:3:end),[],2);
    quad_minX = min(quad_verts(:,1:3:end),[],2);
    quad_minY = min(quad_verts(:,2:3:end),[],2);
    quad_minZ = min(quad_verts(:,3:3:end),[],2);
    % assemble as bounding box
    quad_boxes = [quads(:,1) quad_maxX quad_maxY quad_maxZ quad_minX...
        quad_minY quad_minZ];
    
    % assemble and sort as final polybox
    poly_boxes = [tri_boxes; quad_boxes];
    poly_boxes = sortrows(poly_boxes,1);
    
else
    
    % if homegenous mesh, process all
    % get tri verts
    all_poly_verts = zeros(size(faces,1),(size(faces,2)*3));
    all_poly_verts(:,1:3) = pnts(faces(:,1),:);
    all_poly_verts(:,4:6) = pnts(faces(:,2),:);
    all_poly_verts(:,7:9) = pnts(faces(:,3),:);
    if size(faces,2)==4
        all_poly_verts(:,10:12) = pnts(faces(:,4),:);
    end
    % get all_poly_vert max mins
    all_poly_maxX = max(all_poly_verts(:,1:3:end),[],2);
    all_poly_maxY = max(all_poly_verts(:,2:3:end),[],2);
    all_poly_maxZ = max(all_poly_verts(:,3:3:end),[],2);
    all_poly_minX = min(all_poly_verts(:,1:3:end),[],2);
    all_poly_minY = min(all_poly_verts(:,2:3:end),[],2);
    all_poly_minZ = min(all_poly_verts(:,3:3:end),[],2);
    % assemble as bounding box
    poly_boxes = [searchable_f(:,1) all_poly_maxX all_poly_maxY...
        all_poly_maxZ all_poly_minX all_poly_minY all_poly_minZ];
end

% we will only be searching the bounding boxes in the ray array plane, so
% remove the box elements in the ray dimensions itself
delindex = zeros(1,7);
delindex([ray_dim+1 ray_dim+1+3]) = 1;
poly_boxes(:,logical(delindex)) = [];

for loop1 = 1:fidelity^2;
    
    % get ray
    ray = rays(loop1, :);
    
    % identify intersecting polygons - first stage (bounding box search)
    inter_polys1 = poly_boxes((poly_boxes(:,2)>ray(1) & poly_boxes(:,4)<ray(1)) & ...
        (poly_boxes(:,3)>ray(2) & poly_boxes(:,5)<ray(2)),1);
    
    % if polys found, proceed, check all properly for intersections, and
    % get intersection points if any
    if isempty(inter_polys1)~=1
        
        % prep/reset storage
        inter_points = [0 0 0];
        write2 = 1;
        
        for loop2 = 1:length(inter_polys1)
            
            % get poly
            poly = faces(inter_polys1(loop2), :);
            % trim zeros (tris in a list also containing quads)
            poly(poly==0) = [];
            % get poly verts
            poly_verts = pnts(poly,:);
            
            % secondary detection - does ray intersect poly itself?
            % slice-like methodology: Rationale
            %   - treat ray (point) as slice horizon @ ray's first array dim
            %   - identify polys edges with verts both above and below slice
            %   horizon (only two in quads/tris)
            %   - use simple constant to find point where slice hits edges
            %   - use simple constant to find point where line between slice/edge
            %   interactions == ray.
            %   - if this point is not between slice/edge interactions, ray is
            %   not intersecting with polygon
            
            poly_verts_horproj = poly_verts(:, array_dims(1));
            % get vert numbers above and below, first as index in poly list
            [verts_abovedex,~] = find((poly_verts_horproj>=ray(1))==1);
            [verts_belowdex,~] = find((poly_verts_horproj<=ray(1))==1);
            % then as vert numbers
            verts_above = poly(verts_abovedex);
            verts_below = poly(verts_belowdex);
            
            % use logic on poly vert order to turn into edge pairs
            % if either above or below is a singleton, procedure is simple. The
            % pairs will be the verts on either side in the poly list (poly
            % list is circular, so first and last entries are special cases
            if length(verts_above)==1 || length(verts_below)==1
                if length(verts_above)==1
                    vert1dex = verts_abovedex;
                    vert3dex = verts_abovedex;
                else
                    vert1dex = verts_belowdex;
                    vert3dex = verts_belowdex;
                end
                if vert1dex==1
                    vert2dex = length(poly);
                    vert4dex = 2;
                elseif vert1dex==length(poly)
                    vert2dex = length(poly)-1;
                    vert4dex = 1;
                else
                    vert2dex = vert1dex+1;
                    vert4dex = vert1dex-1;
                end
            else
                % if above and below are binary two things are true -
                % 1, the poly is a quad, 2, it has been sliced in two. No other
                % outcomes are possible because this code will have stopped if
                % anything other than a tri or quad is present. Doesnt matter
                % if we start above or below, so start above (christian-like)
                vert1dex = verts_abovedex(1);
                vert3dex = verts_abovedex(2);
                % the corresponding below vert must be either one up or down on
                % poly list: test (NB circular, first and last special)
                if vert1dex==1
                    % if first is one, second must be either second or last
                    vert2dex = verts_belowdex(verts_belowdex==vert1dex+1 |...
                        verts_belowdex==length(poly));
                elseif vert1dex==length(poly);
                    % if first is last, second must be either second from last
                    % or first
                    vert2dex = verts_belowdex(verts_belowdex==vert1dex-1 |...
                        verts_belowdex==1);
                else
                    % else it must be either up or down one
                    vert2dex = verts_belowdex(verts_belowdex==vert1dex+1 |...
                        verts_belowdex==vert1dex-1);
                end
                if vert3dex==1
                    % if first is one, second must be either second or last
                    vert4dex = verts_belowdex(verts_belowdex==vert3dex+1 |...
                        verts_belowdex==length(poly));
                elseif vert3dex==length(poly);
                    % if first is last, second must be either second from last
                    % or first
                    vert4dex = verts_belowdex(verts_belowdex==vert3dex-1 |...
                        verts_belowdex==1);
                else
                    % else it must be either up or down one
                    vert4dex = verts_belowdex(verts_belowdex==vert3dex+1 |...
                        verts_belowdex==vert3dex-1);
                end
            end
            % now we have two pairs of edges that cross ray horizon
            edge1 = [poly_verts(vert1dex,:); poly_verts(vert2dex,:)];
            edge2 = [poly_verts(vert3dex,:); poly_verts(vert4dex,:)];
            
            % find the points on each edge equal to the ray horizon
            
            % get ratio of size-to-horizon to total edge size in horizon axis
            edge1_factor = (edge1(1, array_dims(1)) - ray(1)) / ...
                (edge1(1, array_dims(1)) - edge1(2, array_dims(1)));
            edge2_factor = (edge2(1, array_dims(1)) - ray(1)) / ...
                (edge2(1, array_dims(1)) - edge2(2, array_dims(1)));
            
            % get vector from edge start to intersect point
            edge1_intervec = (edge1(2,:) - edge1(1,:)) * edge1_factor;
            edge2_intervec = (edge2(2,:) - edge2(1,:)) * edge2_factor;
            
            % get intesect points
            edge1_point = edge1(1,:) + edge1_intervec;
            edge2_point = edge2(1,:) + edge2_intervec;
            
            % INTERSECTION TEST TWO: DO THE TWO POINTS BOUND THE RAY POINT?
            testmat =  sort([edge1_point(array_dims(2)) edge2_point(array_dims(2))]);
            if ray(2)>=testmat(1) && ray(2)<=testmat(2)
                
                % IF SO, FIND INTERSECTION
                ray_factor = (edge1_point(array_dims(2)) - ray(2)) / ...
                    (edge1_point(array_dims(2)) - edge2_point(array_dims(2)));
                ray_intervec = (edge2_point - edge1_point) * ray_factor;
                ray_point = edge1_point + ray_intervec;
                
                % write point
                inter_points(write2,:) = ray_point;
                write2 = write2+1;
                
            end
            
        end
        
        % set proceed (overwrite if condition met)
        proceed = 0;
        
        % ERROR CHECKING - if any intersection points found, check for
        % dupes
        if write2~=1
            
            % sort intersecting points along ray axis
            inter_points = sortrows(inter_points, ray_dim);
            
            % EXCEPTION CATCH: if the ray coincides exactly with a polygon
            % edge, you will get duplication in the interpoints list, which is
            % bad. Simple to remove though.
            % ah.. maybe not THAT simple to remove. unique does not catch
            % everything. Use threshold value instead...
            dupedex = diff(inter_points(:,ray_dim))<identity_threshold;
            inter_points(dupedex,:) = [];
            
            % EXCEPTION CATCH: if the ray only intersects with one edge,
            % i.e just 'grazes' mesh (could happen at very high fidelity)
            % then duplicate check could leave singleton intersect point.
            % check for this, and if it happens, short loop. This can be
            % differentiated from simply hitting part of a non-manifold
            % mesh by the presence of duplicates (i.e. non-zero dupedex
            % entries), as hopefully (hopefully) these will not occur on
            % the same ray
            % reset proceed (overwrite if condition met)
            proceed = 1;
            if sum(dupedex(dupedex==1))~=0
                if mod(size(inter_points,1),2)==1
                    proceed = 0; % do not proceed
                end
            end
        end
        
        % if any intersection points where found, proceed to build cuboids
        if proceed==1
            
            % use intersecting points to build cuboid(s)
            
            % error state - if there are an odd number of intersection points, the
            % ray must entered the mesh and did not leave, i.e. the MESH IS NON
            % MANIFOLD
            if mod(size(inter_points,1),2)==1
                ray_address = zeros(1,3);
                ray_address(array_dims) = ray;
                ray_string = ['(', num2str(ray_address(1)),',', ...
                    num2str(ray_address(2)),',',num2str(ray_address(3)),')'];
                
                % draw mesh
                if sum(sum(faces==0))>0;
                    compoundMesh = 1;
                    [rows,~] = find(faces==0);
                    tris = faces(rows,:);
                    tris(:,4) = [];
                    quads = faces;
                    quads(rows,:) = [];
                else
                    compoundMesh = 0;
                end
                
                figure(1)
                clf
                hold on
                if compoundMesh ==0;
                    trimesh(faces,pnts(:,1), pnts(:,2), pnts(:,3),'edgecolor','m', 'facecolor','none')
                else
                    trimesh(quads,pnts(:,1), pnts(:,2), pnts(:,3),'edgecolor','m', 'facecolor','none')
                    trimesh(tris,pnts(:,1), pnts(:,2), pnts(:,3),'edgecolor','m', 'facecolor','none')
                end
                view ([0,0])
                axis('equal')
                
                % draw ray
                ray_plot = zeros(2,3);
                ray_plot(1,array_dims) = ray;
                ray_plot(2,array_dims) = ray;
                ray_plot(1,ray_dim) = body_box(2,ray_dim)-(ray_dim*0.25);
                ray_plot(2,ray_dim) = body_box(1,ray_dim)+(ray_dim*0.25);
                plot3(ray_plot(:,1), ray_plot(:,2), ray_plot(:,3), '-','color','k')
                text(ray_plot(1,1), ray_plot(1,2), ray_plot(1, 3), num2str(loop1), 'VerticalAlignment', 'top')
                
                title('ERROR IN MESH PROCESSING')
                
                error(['ERROR PROCESSING ',Wobjname,' RAY ',num2str(loop1),' AT ', ray_string,...
                    ' ENTERED MESH AND DID NOT LEAVE, I.E MESH IS NON-MANIFOLD (NOT WATERTIGHT)'])
            end
            
            % break into pairs (inside/outside) that define length of cuboid
            % elements. Other dimensions are determined by the array setup.
            
            start_index = 1:2:size(inter_points,1);
            stop_index = 2:2:size(inter_points,1);
            % get element lengths
            element_lengths = sqrt(sum(abs(inter_points(start_index,:) -  inter_points(stop_index,:)).^2,2));
            
            % matrix-asize-ationate
            length_matrix = zeros(length(element_lengths),3);
            length_matrix(:,ray_dim) = element_lengths/2;
            
            % apppend element CoMs
            TEMP_CoMs = ((inter_points(start_index,:) + length_matrix) + ...
                (inter_points(stop_index,:) - length_matrix))./2;
            
            element_CoMs = [element_CoMs; TEMP_CoMs];
            
            
            % append element masses
            TEMP_masses = element_lengths * array_1_unit * array_2_unit * density;
            element_masses = [element_masses; TEMP_masses];
            
        end
        
    end
    
end

% trim!
element_masses(1) = [];
element_CoMs(1,:) = [];

%% SUM BODY MASS PROPERTIES

% sum mass
body_mass = sum(element_masses);

% calculate CoM
body_CoM = (1/body_mass) * sum(element_CoMs .* ...
    [element_masses element_masses element_masses]);

%% output

cent = body_CoM;

end

function intersects = getRayMeshIntersect(v,f,ray)

% NB ray must be two part vector (start:stop), not just from zero!

% zero so that that start of ray = zero
zero = ray(1,:);
rayTrans = ray(2,:) - zero;

% make unit CS out of ray
rayCS = getCS(rayTrans);

% transform so that ray==X
vTrans = (v-repmat(zero,size(v,1),1))*rayCS';


% find polys with bounding boxes including zero in Y and Z (potential
% polys)
polyBoxes = getPolyBoxes(f,vTrans);
potPolyDex = logical((polyBoxes(:,3)<=0 & polyBoxes(:,4)>=0) &...
    (polyBoxes(:,5)<=0 & polyBoxes(:,6)>=0));
potPolys = f(potPolyDex,:);


if isempty(potPolys);
    intersects = [NaN NaN NaN];
else
    % work out which (if any) polys it's actually hitting...
    intersects = zeros(size(potPolys,1),3);
    for loop1 = 1:size(potPolys,1)
        currPoly = potPolys(loop1,:);
        currPolyV = vTrans(currPoly,:);
        intersects(loop1,:) = getXPolyIntersect(currPolyV);
    end
    % cull NaNs
    intersects(isnan(intersects(:,1)),:) = [];
    
    % transform back
    intersects = (intersects * rayCS) + repmat(zero,size(intersects,1),1);
end

end

function [ polyBoxes ] = getPolyBoxes(pF,pV)

% replace any zero entries with NaN
pF(pF==0) = NaN;

% prepStorage (x y z min max, 6 col)
polyBoxes = zeros(size(pF,1),6);

% populate
for loop1 = 1:size(pF,1)
    currV = pV(pF(loop1,:),:);
    currBox = [min(currV(:,1)) max(currV(:,1))...
        min(currV(:,2)) max(currV(:,2))...
        min(currV(:,3)) max(currV(:,3))];
    polyBoxes(loop1,:) = currBox;
end

end

function [ intersect ] = getXPolyIntersect(pV)
% ok, we want to know whether or not [1 0 0] and the poly intersect. Do
% it with logic and horizon-crossing. As

% get verts as edge pairs. Presume it's a loop, edges go from one vert
% to the next in line. We're in the YZ plane so lets limt our search to
% Z dim only
edgePairs = [pV(:,3) [pV(2:end,3); pV(1,3)]];
edgePairIndices = [(1:size(pV,1))' [(2:size(pV,1))'; 1]];
% two edges must cross zero, otherwise the no intersection
horPair = edgePairIndices(or((edgePairs(:,1)<0 & edgePairs(:,2)>0),...
    (edgePairs(:,1)>0 & edgePairs(:,2)<0)),:);

if isempty(horPair);
    % return NaN
    intersect = [NaN NaN NaN];
else
    
    % ok, we have a potential intersection, use vector proportions to get
    % to z = 0 for both vectors. make vector start below zero. See notebook
    % for details.
    edge1V = sortrows([pV(horPair(1,1),:); pV(horPair(1,2),:)],3);
    edge1Vec = edge1V(2,:) - edge1V(1,:);
    edge1inter = edge1V(1,:)+(edge1Vec.*(abs(edge1V(1,3))/abs(edge1Vec(3))));
    
    edge2V = sortrows([pV(horPair(2,1),:); pV(horPair(2,2),:)],3);
    edge2Vec = edge2V(2,:) - edge2V(1,:);
    edge2inter = edge2V(1,:)+(edge2Vec.*(abs(edge2V(1,3))/abs(edge2Vec(3))));
    
    % does vector between two intersects include y==0?
    if ~or((edge1inter(2)<0 && edge2inter(2)>0),(edge1inter(2)>0 && edge2inter(2)<0))
        % return NaN
        intersect = [NaN NaN NaN];
    else
        % get and return intersect point (y==0)
        edgeintersects = sortrows([edge1inter; edge2inter],2);
        edgesVec = edgeintersects(2,:) - edgeintersects(1,:);
        intersect = edgeintersects(1,:)+(edgesVec.*(abs(edgeintersects(1,2))/abs(edgesVec(2))));
    end
    
end

end
